@@ -20,6 +20,8 @@ import { OpenAPIV3 } from "openapi-types";
 import * as parseOpenapi from "parse-openapi";
 import { getOperationResponses } from "parse-openapi/dist/parser/getOperationResponses";
 import { getOperationResponse } from "parse-openapi/dist/parser/getOperationResponse";
+import { generateMockDataForSchema } from "../custom/mock-data/generate-mock-data";
+import { allFakers, Faker } from "@faker-js/faker";
 
 const TSAPI_WRITE_FILE_START = "###TSAPI_WRITE_FILE###";
 const TSAPI_WRITE_FILE_END = "###/TSAPI_WRITE_FILE###";
@@ -111,6 +113,106 @@ const resolveIfRef = <T>(spec: OpenAPIV3.Document, possibleRef: T | OpenAPIV3.Re
   return resolved as T;
 };
 
+/**
+ * Copy vendor extensions from the first parameter to the second
+ */
+const copyVendorExtensions = (object: object, vendorExtensions: { [key: string]: any }) => {
+  Object.entries(object ?? {}).forEach(([key, value]) => {
+    if (key.startsWith('x-')) {
+      vendorExtensions[key] = value;
+    }
+  });
+};
+
+/**
+ * Converts a string to snake_case. Matches OpenAPI generator's behaviour, which is slightly different to lodash
+ * in how it handles numbers (ie no underscore before a number, but an underscore after a number so long as the next letter is capitalized)
+ * @see https://github.com/OpenAPITools/openapi-generator/blob/38dac13c261d26a72be78bba89ee4a681843e7b0/modules/openapi-generator/src/main/java/org/openapitools/codegen/utils/StringUtils.java#L77
+ */
+const snakeCase = (str: string): string => {
+  return str
+    .replace(/\./g, '/')
+    .replace(/\$/g, '__')
+    .replace(/([A-Z]+)([A-Z][a-z][a-z]+)/g, (_match, g1, g2) => `${g1}_${g2}`)
+    .replace(/([a-z\d])([A-Z])/g, (_match, g1, g2) => `${g1}_${g2}`)
+    .replace(/[-\s]/g, '_')
+    .toLowerCase();
+};
+
+// @see https://github.com/OpenAPITools/openapi-generator/blob/e2a62ace74de361bef6338b7fa37da8577242aef/modules/openapi-generator/src/main/java/org/openapitools/codegen/languages/AbstractPythonCodegen.java#L106
+const PYTHON_KEYWORDS = new Set([
+  // local variable name used in API methods (endpoints)
+  "all_params", "resource_path", "path_params", "query_params",
+  "header_params", "form_params", "local_var_files", "body_params", "auth_settings",
+  // @property
+  "property",
+  // typing keywords
+  "schema", "base64", "json",
+  "date", "float",
+  // python reserved words
+  "and", "del", "from", "not", "while", "as", "elif", "global", "or", "with",
+  "assert", "else", "if", "pass", "yield", "break", "except", "import",
+  "print", "class", "exec", "in", "raise", "continue", "finally", "is",
+  "return", "def", "for", "lambda", "try", "self", "nonlocal", "None", "True",
+  "False", "async", "await",
+]);
+
+const toPythonName = (namedEntity: 'model' | 'property' | 'operation', name: string) => {
+  const nameSnakeCase = snakeCase(name);
+
+  // Check if the name is a reserved word. Reserved words that overlap with TypeScript will already be escaped
+  // with a leading _ by parseOpenapi, so we remove this to test
+  if (PYTHON_KEYWORDS.has(name.startsWith('_') ? name.slice(1) : name)) {
+    const nameSuffix = name.startsWith('_') ? nameSnakeCase : `_${nameSnakeCase}`;
+    switch(namedEntity) {
+      case "model":
+        return `model${nameSuffix}`;
+      case "operation":
+        return `call${nameSuffix}`;
+      case "property":
+        return `var${nameSnakeCase}`;
+      default:
+        break;
+    }
+  }
+  return nameSnakeCase;
+};
+
+// @see https://github.com/OpenAPITools/openapi-generator/blob/8f2676c5c2bcbcc41942307e5c8648cee38bcc44/modules/openapi-generator/src/main/java/org/openapitools/codegen/languages/AbstractJavaCodegen.java#L179
+const JAVA_KEYWORDS = new Set([
+  // special words
+  "object", "list", "file",
+  // used as internal variables, can collide with parameter names
+  "localVarPath", "localVarQueryParams", "localVarCollectionQueryParams",
+  "localVarHeaderParams", "localVarCookieParams", "localVarFormParams", "localVarPostBody",
+  "localVarAccepts", "localVarAccept", "localVarContentTypes",
+  "localVarContentType", "localVarAuthNames", "localReturnType",
+  "ApiClient", "ApiException", "ApiResponse", "Configuration", "StringUtil",
+
+  // language reserved words
+  "_", "abstract", "continue", "for", "new", "switch", "assert",
+  "default", "if", "package", "synchronized", "boolean", "do", "goto", "private",
+  "this", "break", "double", "implements", "protected", "throw", "byte", "else",
+  "import", "public", "throws", "case", "enum", "instanceof", "return", "transient",
+  "catch", "extends", "int", "short", "try", "char", "final", "interface", "static",
+  "void", "class", "finally", "long", "strictfp", "volatile", "const", "float",
+  "native", "super", "while", "null", "offsetdatetime", "localdate", "localtime"
+]);
+
+const toJavaName = (name: string) => {
+  // Check if the name is a reserved word. Reserved words that overlap with TypeScript will already be escaped
+  // with a leading _ by parseOpenapi, so we remove this to test
+  const unescapedName = _camelCase(name.startsWith('_') ? name.slice(1) : name);
+  if (JAVA_KEYWORDS.has(unescapedName)) {
+    // Special case for "class"
+    if (unescapedName === "class") {
+      return "propertyClass";
+    }
+    return `_${unescapedName}`;
+  }
+  return unescapedName;
+};
+
 /**
  * Clean up any generated code that already exists
  */
@@ -201,23 +303,39 @@ const splitAndWriteFiles = (renderedFileContents: string[], outputPath: string)
 const COMPOSED_SCHEMA_TYPES = new Set(["one-of", "any-of", "all-of"]);
 const PRIMITIVE_TYPES = new Set(["string", "integer", "number", "boolean", "null", "any", "binary", "void"]);
 
-const resolveComposedProperties = (data: parseOpenapi.ParsedSpecification, model: parseOpenapi.Model): parseOpenapi.Model[] => {
-  if (COMPOSED_SCHEMA_TYPES.has(model.export)) {
-    // Find the composed model, if any
-    const parentModelProperty = model.properties.find(p => !p.name && p.export === "reference");
-    const composedModel = data.models.find(m => m.name === parentModelProperty?.type);
+/**
+ * Mutates the given data to ensure composite models (ie allOf, oneOf, anyOf) have the necessary
+ * properties for representing them in generated code. Adds `composedModels` and `composedPrimitives`
+ * which contain the models and primitive types that each model is composed of.
+ */
+const ensureCompositeModels = (data: parseOpenapi.ParsedSpecification) => {
+  const visited = new Set<parseOpenapi.Model>();
+  data.models.forEach(model => mutateModelWithCompositeProperties(data, model, visited));
+}
 
-    // The actual properties for a composed model are nested under a special property called "properties"
-    return [
-      ...(model.properties.find(p => p.name === "properties")?.properties ?? []),
-      ...(composedModel ? resolveComposedProperties(data, composedModel).map((p => {
-        (p as any)._composedFrom = composedModel.name;
-        return p;
-      })) : []),
-    ];
+const mutateModelWithCompositeProperties = (data: parseOpenapi.ParsedSpecification, model: parseOpenapi.Model, visited: Set<parseOpenapi.Model>) => {
+  if (COMPOSED_SCHEMA_TYPES.has(model.export) && !visited.has(model)) {
+    visited.add(model);
+
+    // Find the models/primitives which this is composed from
+    const composedModelReferences = model.properties.filter(p => !p.name && p.export === "reference");
+    const composedPrimitives = model.properties.filter(p => !p.name && p.export !== "reference");
+
+    const modelsByName = Object.fromEntries(data.models.map(m => [m.name, m]));
+    const composedModels = composedModelReferences.flatMap(r => modelsByName[r.type] ? [modelsByName[r.type]] : []);
+    // Recursively resolve composed properties of properties, to ensure mixins for all-of include all recursive all-of properties
+    composedModels.forEach(m => mutateModelWithCompositeProperties(data, m, visited));
+
+    // For all-of models, we include all composed model properties.
+    if (model.export === "all-of") {
+      if (composedPrimitives.length > 0) {
+        throw new Error(`Schema "${model.name}" defines allOf with non-object types. allOf may only compose object types in the OpenAPI specification.`);
+      }
+    }
+
+    (model as any).composedModels = composedModels;
+    (model as any).composedPrimitives = composedPrimitives;
   }
-
-  return model.properties ?? [];
 };
 
 const toTypescriptPrimitive = (property: parseOpenapi.Model): string => {
@@ -238,25 +356,31 @@ const toTypeScriptType = (property: parseOpenapi.Model): string => {
     case "reference":
       return toTypescriptPrimitive(property);
     case "array":
-      return `Array<${property.link ? toTypeScriptType(property.link) : property.type}>`;
+      return `Array<${property.link && property.link.export !== "enum" ? toTypeScriptType(property.link) : property.type}>`;
     case "dictionary":
-      return `{ [key: string]: ${property.link ? toTypeScriptType(property.link) : property.type}; }`;
+      return `{ [key: string]: ${property.link && property.link.export !== "enum" ? toTypeScriptType(property.link) : property.type}; }`;
     default:
       return property.type;
   }
 };
 
 const toJavaPrimitive = (property: parseOpenapi.Model): string => {
-  if (property.type === "string" && ["date", "date-time"].includes(property.format ?? '')) {
-    return "Date";
-  } else if (property.type === "binary") {
+  if (property.type === "string" && property.format === "date") {
+    return "LocalDate";
+  } else if (property.type === "string" && property.format === "date-time") {
+    return "OffsetDateTime";
+  } else if (property.type === "string" && (property.format as any) === "uuid") {
+    return "UUID";
+  } else if (property.type === "string" && (property.format as any) === "uri") {
+    return "URI";
+  } else if (property.type === "binary" || (property.type === "string" && ["byte", "binary"].includes(property.format as any))) {
     return "byte[]";
   } else if (property.type === "number") {
     switch(property.format) {
       case "int32":
         return "Integer";
       case "int64":
-        return "BigInteger";
+        return "Long";
       case "float":
         return "Float";
       case "double":
@@ -273,6 +397,8 @@ const toJavaPrimitive = (property: parseOpenapi.Model): string => {
     return "Boolean";
   } else if (property.type === "string") {
     return "String";
+  } else if (property.type === "any") {
+    return "Object";
   }
   return property.type;
 };
@@ -283,10 +409,63 @@ const toJavaType = (property: parseOpenapi.Model): string => {
     case "reference":
       return toJavaPrimitive(property);
     case "array":
-      return `${property.uniqueItems ? 'Set' : 'List'}<${property.link ? toTypeScriptType(property.link) : property.type}>`;
+      return `${property.uniqueItems ? 'Set' : 'List'}<${property.link && property.link.export !== "enum" ? toJavaType(property.link) : property.type}>`;
     case "dictionary":
-      return `Map<String, ${property.link ? toTypeScriptType(property.link) : property.type}>`;
+      return `Map<String, ${property.link && property.link.export !== "enum" ? toJavaType(property.link) : property.type}>`;
     default:
+      // "any" has export = interface
+      if (PRIMITIVE_TYPES.has(property.type)) {
+        return toJavaPrimitive(property);
+      }
+      return property.type;
+  }
+};
+
+const toPythonPrimitive = (property: parseOpenapi.Model): string => {
+  if (property.type === "string" && property.format === "date") {
+    return "date";
+  } else if (property.type === "string" && property.format === "date-time") {
+    return "datetime"
+  } else if (property.type === "any") {
+    return "object";
+  } else if (property.type === "binary") {
+    return "bytearray";
+  } else if (property.type === "number") {
+    if ((property as any).openapiType === "integer") {
+      return "int";
+    }
+
+    switch(property.format) {
+      case "int32":
+      case "int64":
+        return "int";
+      case "float":
+      case "double":
+      default:
+        return "float";
+    }
+  } else if (property.type === "boolean") {
+    return "bool";
+  } else if (property.type === "string") {
+    return "str";
+  }
+  return property.type;
+};
+
+const toPythonType = (property: parseOpenapi.Model): string => {
+  switch (property.export) {
+    case "generic":
+    case "reference":
+      return toPythonPrimitive(property);
+    case "array":
+      return `List[${property.link && property.link.export !== "enum" ? toPythonType(property.link) : property.type}]`;
+    case "dictionary":
+      return `Dict[str, ${property.link && property.link.export !== "enum" ? toPythonType(property.link) : property.type}]`;
+    default:
+      // "any" has export = interface
+      if (PRIMITIVE_TYPES.has(property.type)) {
+        return toPythonPrimitive(property);
+      }
       return property.type;
   }
 };
@@ -295,30 +474,156 @@ const toJavaType = (property: parseOpenapi.Model): string => {
  * Mutates the given model to add language specific types and names
  */
 const mutateModelWithAdditionalTypes = (model: parseOpenapi.Model) => {
-  (model as any).typescriptName = model.name;
-  (model as any).typescriptType = toTypeScriptType(model);
-  (model as any).javaType = toJavaType(model);
-  (model as any).isPrimitive = PRIMITIVE_TYPES.has(model.type);
-
   // Trim any surrounding quotes from name
   model.name = _trim(model.name, `"'`);
+
+  (model as any).typescriptName = model.name;
+  (model as any).typescriptType = toTypeScriptType(model);
+  (model as any).javaName = toJavaName(model.name);
+  (model as any).javaType = toJavaType(model);
+  (model as any).pythonName = toPythonName('property', model.name);
+  (model as any).pythonType = toPythonType(model);
+  (model as any).isPrimitive = PRIMITIVE_TYPES.has(model.type);
 };
 
-const mutateWithOpenapiSchemaProperties = (spec: OpenAPIV3.Document, model: parseOpenapi.Model, schema: OpenAPIV3.SchemaObject, visited: Set<parseOpenapi.Model> = new Set()) => {
+interface MockDataContext {
+  readonly dereferencedSpec: OpenAPIV3.Document;
+  readonly faker: Faker;
+}
+
+const mutateWithOpenapiSchemaProperties = (spec: OpenAPIV3.Document, model: parseOpenapi.Model, schema: OpenAPIV3.SchemaObject, mockDataContext: MockDataContext, visited: Set<parseOpenapi.Model> = new Set()) => {
   (model as any).format = schema.format;
-  (model as any).oapiType = schema.type;
   (model as any).isInteger = schema.type === "integer";
   (model as any).isShort = schema.format === "int32";
   (model as any).isLong = schema.format === "int64";
   (model as any).deprecated = !!schema.deprecated;
   (model as any).openapiType = schema.type;
+  (model as any).isNotSchema = !!schema.not;
+  (model as any).isEnum = !!schema.enum && schema.enum.length > 0;
+
+  // Copy any schema vendor extensions
+  (model as any).vendorExtensions = {};
+  copyVendorExtensions(schema, (model as any).vendorExtensions);
+
+  // Use our added vendor extension
+  (model as any).isHoisted = !!(model as any).vendorExtensions?.['x-tsapi-hoisted'];
+
+  mutateModelWithAdditionalTypes(model);
+
+  // Add mock data
+  (model as any).mockData = generateMockDataForSchema(mockDataContext.dereferencedSpec, {
+    faker: mockDataContext.faker,
+    maxArrayLength: 3,
+    maxCircularReferenceDepth: 2,
+  }, schema);
 
   visited.add(model);
 
   // Also apply to array items recursively
   if (model.export === "array" && model.link && 'items' in schema && schema.items && !visited.has(model.link)) {
     const subSchema = resolveIfRef(spec, schema.items);
-    mutateWithOpenapiSchemaProperties(spec, model.link, subSchema, visited);
+    mutateWithOpenapiSchemaProperties(spec, model.link, subSchema, mockDataContext, visited);
+  }
+
+  // Also apply to object properties recursively
+  if (model.export === "dictionary" && model.link && 'additionalProperties' in schema && schema.additionalProperties && !visited.has(model.link)) {
+    const subSchema = resolveIfRef(spec, schema.additionalProperties);
+    // Additional properties can be "true" rather than a type
+    if (subSchema !== true) {
+      mutateWithOpenapiSchemaProperties(spec, model.link, subSchema, mockDataContext, visited);
+    }
+  }
+  model.properties.filter(p => !visited.has(p) && schema.properties?.[_trim(p.name, `"'`)]).forEach(property => {
+    const subSchema = resolveIfRef(spec, schema.properties![_trim(property.name, `"'`)]);
+    mutateWithOpenapiSchemaProperties(spec, property, subSchema, mockDataContext, visited);
+  });
+
+  if (COMPOSED_SCHEMA_TYPES.has(model.export)) {
+    model.properties.forEach((property, i) => {
+      const subSchema = resolveIfRef(spec, (schema as any)[_camelCase(model.export)]?.[i]);
+      if (subSchema) {
+        mutateWithOpenapiSchemaProperties(spec, property, subSchema, mockDataContext, visited);
+      }
+    });
+  }
+};
+
+/**
+ * Ensure that the "link" property of all dictionary/array models and properties are set recursively
+ */
+const ensureModelLinks = (spec: OpenAPIV3.Document, data: parseOpenapi.ParsedSpecification) => {
+  const modelsByName = Object.fromEntries(data.models.map((m) => [m.name, m]));
+  const visited = new Set<parseOpenapi.Model>();
+
+  // Ensure set for all models
+  data.models.forEach((model) => {
+    const schema = resolveIfRef(spec, spec?.components?.schemas?.[model.name]);
+    if (schema) {
+      // Object schemas should be typed as the model we will create
+      if (schema.type === "object" && schema.properties) {
+        model.type = model.name;
+      }
+      _ensureModelLinks(spec, modelsByName, model, schema, visited)
+    }
+  });
+
+  // Ensure set for all parameters
+  data.services.forEach((service) => {
+    service.operations.forEach((op) => {
+      const specOp = (spec as any)?.paths?.[op.path]?.[op.method.toLowerCase()] as OpenAPIV3.OperationObject | undefined;
+
+      const specParametersByName = Object.fromEntries((specOp?.parameters ?? []).map((p) => {
+        const param = resolveIfRef(spec, p);
+        return [param.name, param];
+      }));
+
+      op.parameters.forEach((parameter) => {
+        const specParameter = specParametersByName[parameter.prop];
+        const specParameterSchema = resolveIfRef(spec, specParameter?.schema);
+
+        if (specParameterSchema) {
+          _ensureModelLinks(spec, modelsByName, parameter, specParameterSchema, visited);
+        }
+      });
+    });
+  });
+};
+
+const _ensureModelLinks = (spec: OpenAPIV3.Document, modelsByName: {[name: string]: parseOpenapi.Model}, model: parseOpenapi.Model, schema: OpenAPIV3.SchemaObject, visited: Set<parseOpenapi.Model>) => {
+  if (visited.has(model)) {
+    return;
+  }
+
+  visited.add(model);
+
+  if (model.export === "dictionary" && 'additionalProperties' in schema && schema.additionalProperties) {
+    if (isRef(schema.additionalProperties)) {
+      const name = splitRef(schema.additionalProperties.$ref)[2];
+      if (modelsByName[name] && !model.link) {
+        model.link = modelsByName[name];
+      }
+    }
+  } else if (model.export === "array" && 'items' in schema && schema.items) {
+    if (isRef(schema.items)) {
+      const name = splitRef(schema.items.$ref)[2];
+      if (modelsByName[name] && !model.link) {
+        model.link = modelsByName[name];
+      }
+    }
+  }
+
+  model.properties.filter(p => !visited.has(p) && schema.properties?.[_trim(p.name, `"'`)]).forEach(property => {
+    const subSchema = resolveIfRef(spec, schema.properties![_trim(property.name, `"'`)]);
+    _ensureModelLinks(spec, modelsByName, property, subSchema, visited);
+  });
+
+  if (COMPOSED_SCHEMA_TYPES.has(model.export)) {
+    model.properties.forEach((property, i) => {
+      const subSchema = resolveIfRef(spec, (schema as any)[_camelCase(model.export)]?.[i]);
+      if (subSchema) {
+        _ensureModelLinks(spec, modelsByName, property, subSchema, visited);
+      }
+    });
   }
 };
 
@@ -334,31 +639,50 @@ interface SubSchemaRef {
   readonly schema: OpenAPIV3.SchemaObject;
 }
 
-const isInlineObjectOrArraySubSchema = (schema?: OpenAPIV3.SchemaObject | OpenAPIV3.ReferenceObject): schema is OpenAPIV3.SchemaObject => !!schema && !isRef(schema) && ["object", "array"].includes(schema.type as any);
+const isCompositeSchema = (schema: OpenAPIV3.SchemaObject) =>
+  !!schema.allOf || !!schema.anyOf || !!schema.oneOf;
+
+const hasSubSchemasToVisit = (schema?: OpenAPIV3.SchemaObject | OpenAPIV3.ReferenceObject): schema is OpenAPIV3.SchemaObject =>
+  !!schema && !isRef(schema) && (["object", "array"].includes(schema.type as any) || isCompositeSchema(schema) || !!schema.not || (schema.type === "string" && !!schema.enum));
+
+const filterInlineCompositeSchemas = (schemas: (OpenAPIV3.SchemaObject | OpenAPIV3.ReferenceObject)[], nameParts: string[], namePartPrefix: string, prop: string): SubSchema[] => {
+  let inlineSchemaIndex = 0;
+  return schemas.flatMap((s, i) => {
+    if (hasSubSchemasToVisit(s)) {
+      const subSchema: SubSchema = { nameParts: [...nameParts, `${namePartPrefix}${inlineSchemaIndex === 0 ? '' : inlineSchemaIndex}`], schema: s, prop: `${prop}.[${i}]` };
+      inlineSchemaIndex++;
+      return [subSchema];
+    }
+    return [];
+  });
+}
 
 const hoistInlineObjectSubSchemas = (nameParts: string[], schema: OpenAPIV3.SchemaObject): SubSchemaRef[] => {
-  // Find all the inline subschemas
+  // Find all the inline subschemas we should visit
   const inlineSubSchemas: SubSchema[] = [
-    ...(isInlineObjectOrArraySubSchema(schema.not) ? [{ nameParts: [...nameParts, 'Not'], schema: schema.not, prop: 'not' }] : []),
-    ...(schema.anyOf ? schema.anyOf.filter(isInlineObjectOrArraySubSchema).map((s, i) => ({ nameParts: [...nameParts, `${i}`], schema: s, prop: `anyOf.[${i}]` })) : []),
-    ...(schema.allOf ? schema.allOf.filter(isInlineObjectOrArraySubSchema).map((s, i) => ({ nameParts: [...nameParts, `${i}`], schema: s, prop: `allOf.[${i}]` })) : []),
-    ...(schema.oneOf ? schema.oneOf.filter(isInlineObjectOrArraySubSchema).map((s, i) => ({ nameParts: [...nameParts, `${i}`], schema: s, prop: `oneOf.[${i}]` })) : []),
-    ...('items' in schema && isInlineObjectOrArraySubSchema(schema.items) ? [{ nameParts: [...nameParts, 'Inner'], schema: schema.items, prop: 'items' }] : []),
-    ...(Object.entries(schema.properties ?? {}).filter(([, s]) => isInlineObjectOrArraySubSchema(s)).map(([name, s]) => ({ nameParts: [...nameParts, name], schema: s as OpenAPIV3.SchemaObject, prop: `properties.${name}` }))),
-    ...((typeof schema.additionalProperties !== "boolean" && isInlineObjectOrArraySubSchema(schema.additionalProperties)) ? [{ nameParts: [...nameParts, 'Value'], schema: schema.additionalProperties, prop: `additionalProperties` }] : []),
+    ...(hasSubSchemasToVisit(schema.not) ? [{ nameParts: [...nameParts, 'Not'], schema: schema.not, prop: 'not' }] : []),
+    ...(schema.anyOf ? filterInlineCompositeSchemas(schema.anyOf, nameParts, 'AnyOf', 'anyOf') : []),
+    ...(schema.allOf ? filterInlineCompositeSchemas(schema.allOf, nameParts, 'AllOf', 'allOf') : []),
+    ...(schema.oneOf ? filterInlineCompositeSchemas(schema.oneOf, nameParts, 'OneOf', 'oneOf') : []),
+    ...('items' in schema && hasSubSchemasToVisit(schema.items) ? [{ nameParts: [...nameParts, 'Inner'], schema: schema.items, prop: 'items' }] : []),
+    ...(Object.entries(schema.properties ?? {}).filter(([, s]) => hasSubSchemasToVisit(s)).map(([name, s]) => ({ nameParts: [...nameParts, name], schema: s as OpenAPIV3.SchemaObject, prop: `properties.${name}` }))),
+    ...((typeof schema.additionalProperties !== "boolean" && hasSubSchemasToVisit(schema.additionalProperties)) ? [{ nameParts: [...nameParts, 'Value'], schema: schema.additionalProperties, prop: `additionalProperties` }] : []),
   ];
 
   // Hoist these recursively first (ie depth first search) so that we don't miss refs
   const recursiveRefs = inlineSubSchemas.flatMap((s) => hoistInlineObjectSubSchemas(s.nameParts, s.schema));
 
-  // Clone the object subschemas to build the refs
-  const refs = inlineSubSchemas.filter(s => s.schema.type === "object" && ["items", "additionalProperties"].includes(s.prop)).map(s => {
-    const name = s.nameParts.map(_upperFirst).join('');
+  // Clone the object subschemas to build the refs. Note that only objects with "properties" are hoisted as these are non-dictionary types
+  const refs = inlineSubSchemas.filter(s => (s.schema.type === "object" && s.schema.properties) || isCompositeSchema(s.schema) || (s.schema.type === "string" && s.schema.enum)).map(s => {
+    const name = [...s.nameParts, ...(s.schema.type === "string" && s.schema.enum ? ["Enum"] : [])].map(_upperFirst).join('');
     const $ref = `#/components/schemas/${name}`;
     const ref = {
       $ref,
       name,
-      schema: structuredClone(s.schema),
+      schema: structuredClone({
+        ...s.schema,
+        "x-tsapi-hoisted": true,
+      }),
     };
 
     // Replace each subschema with a ref in the spec
@@ -370,13 +694,13 @@ const hoistInlineObjectSubSchemas = (nameParts: string[], schema: OpenAPIV3.Sche
   return [...refs, ...recursiveRefs];
 };
 
-const buildData = (inSpec: OpenAPIV3.Document, metadata: any) => {
+const buildData = async (inSpec: OpenAPIV3.Document, metadata: any) => {
   // Using openapi generator, we passed "KEEP_ONLY_FIRST_TAG_IN_OPERATION" to ensure we don't generate duplicate
   // handler wrappers where multiple tags are used.
   // In order for the new generator not to be breaking, we apply the same logic here, however this can be removed
   // in future since we have control to avoid the duplicate handlers while allowing an operation to be part of
   // multiple "services".
-  const spec = JSON.parse(JSON.stringify(inSpec, (key, value) => {
+  let spec = JSON.parse(JSON.stringify(inSpec, (key, value) => {
     // Keep only the first tag where we find a tag
     if (key === "tags" && value && value.length > 0 && typeof value[0] === "string") {
       return [value[0]];
@@ -423,9 +747,44 @@ const buildData = (inSpec: OpenAPIV3.Document, metadata: any) => {
     }
   });
 
+  // "Inline" any refs to non objects/enums
+  const inlinedRefs: Set<string> = new Set();
+  spec = JSON.parse(JSON.stringify(spec, (k, v) => {
+    if (v && typeof v === "object" && v.$ref) {
+      const resolved = resolveRef(spec, v.$ref);
+      if (resolved && resolved.type && resolved.type !== "object" && !(resolved.type === "string" && resolved.enum)) {
+        inlinedRefs.add(v.$ref);
+        return resolved;
+      }
+    }
+    return v;
+  }));
+
+  // Delete the non object schemas that were inlined
+  [...inlinedRefs].forEach(ref => {
+    const parts = splitRef(ref);
+    if (parts.length === 3 && parts[0] === "components" && parts[1] === "schemas") {
+      delete spec.components!.schemas![parts[2]];
+    }
+  });
+
   // Start with the data from https://github.com/webpro/parse-openapi which extracts most of what we need
   const data = { ...parseOpenapi.parse(spec), metadata };
 
+  // Mutate the models with enough data to render composite models in the templates
+  ensureCompositeModels(data);
+
+  // Ensure the models have their links set when they are arrays/dictionaries
+  ensureModelLinks(spec, data);
+
+  const faker = allFakers['en'];
+  faker.seed(1337);
+  faker.setDefaultRefDate(new Date("2021-06-10"));
+  const mockDataContext: MockDataContext = {
+    faker,
+    dereferencedSpec: await SwaggerParser.dereference(structuredClone(spec), { dereference: { circular: 'ignore' }}) as OpenAPIV3.Document,
+  };
+
   // Augment operations with additional data
   data.services.forEach((service) => {
 
@@ -438,12 +797,8 @@ const buildData = (inSpec: OpenAPIV3.Document, metadata: any) => {
       const specOp = (spec as any)?.paths?.[op.path]?.[op.method.toLowerCase()] as OpenAPIV3.OperationObject | undefined;
 
       // Add vendor extensions
-      Object.entries(specOp ?? {}).forEach(([key, value]) => {
-        if (key.startsWith('x-')) {
-          (op as any).vendorExtensions = (op as any).vendorExtensions ?? {};
-          (op as any).vendorExtensions[key] = value;
-        }
-      });
+      (op as any).vendorExtensions = (op as any).vendorExtensions ?? {};
+      copyVendorExtensions(specOp ?? {}, (op as any).vendorExtensions);
 
       if (specOp) {
         // parseOpenapi has a method to retrieve the operation responses, but later filters to only
@@ -483,6 +838,11 @@ const buildData = (inSpec: OpenAPIV3.Document, metadata: any) => {
             } else {
               // Add the response media types
               (response as any).mediaTypes = Object.keys(specResponse.content);
+
+              const responseSchema = specResponse.content?.['application/json'] ?? Object.values(specResponse.content)[0];
+              if (responseSchema) {
+                mutateWithOpenapiSchemaProperties(spec, response, responseSchema, mockDataContext);
+              }
             }
           }
         });
@@ -510,7 +870,7 @@ const buildData = (inSpec: OpenAPIV3.Document, metadata: any) => {
         const specParameterSchema = resolveIfRef(spec, specParameter?.schema);
 
         if (specParameterSchema) {
-          mutateWithOpenapiSchemaProperties(spec, parameter, specParameterSchema);
+          mutateWithOpenapiSchemaProperties(spec, parameter, specParameterSchema, mockDataContext);
         }
 
         if (parameter.in === "body") {
@@ -525,7 +885,7 @@ const buildData = (inSpec: OpenAPIV3.Document, metadata: any) => {
             if (parameter.mediaType) {
               const bodySchema = resolveIfRef(spec, specBody.content?.[parameter.mediaType]?.schema);
               if (bodySchema) {
-                mutateWithOpenapiSchemaProperties(spec, parameter, bodySchema);
+                mutateWithOpenapiSchemaProperties(spec, parameter, bodySchema, mockDataContext);
               }
             }
             // Track all the media types that can be accepted in the request body
@@ -553,7 +913,7 @@ const buildData = (inSpec: OpenAPIV3.Document, metadata: any) => {
       // Add variants of operation name
       (op as any).operationIdPascalCase = _upperFirst(op.name);
       (op as any).operationIdKebabCase = _kebabCase(op.name);
-      (op as any).operationIdSnakeCase = _snakeCase(op.name);
+      (op as any).operationIdSnakeCase = toPythonName('operation', op.name);
     });
 
     // Lexicographical ordering of operations to match openapi generator
@@ -564,24 +924,27 @@ const buildData = (inSpec: OpenAPIV3.Document, metadata: any) => {
 
     // Add the service class name
     (service as any).className = `${service.name}Api`;
+    (service as any).classNameSnakeCase = snakeCase((service as any).className);
+    (service as any).nameSnakeCase = snakeCase(service.name);
   });
 
   // Augment models with additional data
   data.models.forEach((model) => {
-    const matchingSpecModel = spec?.components?.schemas?.[model.name]
+    // Add a snake_case name
+    (model as any).nameSnakeCase = toPythonName('model', model.name);
+
+    const matchingSpecModel = spec?.components?.schemas?.[model.name];
 
     if (matchingSpecModel) {
-      const specModel = isRef(matchingSpecModel) ? resolveRef(spec, matchingSpecModel.$ref) as OpenAPIV3.SchemaObject : matchingSpecModel;
+      const specModel = resolveIfRef(spec, matchingSpecModel);
 
-      // Resolve properties inherited from composed schemas
-      const composedProperties = resolveComposedProperties(data, model);
-      (model as any).resolvedProperties = composedProperties;
+      mutateWithOpenapiSchemaProperties(spec, model, specModel, mockDataContext);
 
       // Add unique imports
       (model as any).uniqueImports = _orderBy(_uniq([
         ...model.imports,
-        // Include composed property imports, if any
-        ...composedProperties.filter(p => p.export === "reference").map(p => p.type),
+        // Include property imports, if any
+        ...model.properties.filter(p => p.export === "reference").map(p => p.type),
       ]));
 
       // Add deprecated flag if present
@@ -593,12 +956,12 @@ const buildData = (inSpec: OpenAPIV3.Document, metadata: any) => {
       }
 
       // Augment properties with additional data
-      [...model.properties, ...((model as any).resolvedProperties as parseOpenapi.Model[])].forEach((property) => {
+      model.properties.forEach((property) => {
         const matchingSpecProperty = specModel.properties?.[property.name];
 
         if (matchingSpecProperty) {
           const specProperty = resolveIfRef(spec, matchingSpecProperty);
-          mutateWithOpenapiSchemaProperties(spec, property, specProperty);
+          mutateWithOpenapiSchemaProperties(spec, property, specProperty, mockDataContext);
         }
 
         // Add language-specific names/types
@@ -618,11 +981,7 @@ const buildData = (inSpec: OpenAPIV3.Document, metadata: any) => {
 
   // Add top level vendor extensions
   const vendorExtensions: { [key: string]: any } = {};
-  Object.entries(spec ?? {}).forEach(([key, value]) => {
-    if (key.startsWith('x-')) {
-      vendorExtensions[key] = value;
-    }
-  });
+  copyVendorExtensions(spec ?? {}, vendorExtensions);
 
   return {
     ...data,
@@ -659,7 +1018,7 @@ export default async (argv: string[], rootScriptDir: string) => {
   const spec = (await SwaggerParser.bundle(args.specPath)) as any;
 
   // Build data
-  const data = buildData(spec, JSON.parse(args.metadata ?? '{}'));
+  const data = await buildData(spec, JSON.parse(args.metadata ?? '{}'));
 
   if (args.printData) {
     console.log(JSON.stringify(data, null, 2));
@@ -669,13 +1028,12 @@ export default async (argv: string[], rootScriptDir: string) => {
   const templates = args.templateDirs.flatMap(t => fs.readdirSync(resolveTemplateDir(rootScriptDir, t), {
     recursive: true,
     withFileTypes: true
-  }).filter(f => f.isFile() && f.name.endsWith('.ejs')).map(f => path.join(f.parentPath ?? f.path, f.name)));
+  }).filter(f => f.isFile() && f.name.endsWith('.ejs') && !f.name.endsWith('.partial.ejs')).map(f => path.join(f.parentPath ?? f.path, f.name)));
 
   // Render the templates with the data from the spec
-  const renderedFiles = templates.map((template) => {
-    const templateContents = fs.readFileSync(template, 'utf-8');
-    return ejs.render(templateContents, data);
-  });
+  const renderedFiles = await Promise.all(templates.map(async (template) => {
+    return await ejs.renderFile(template, data);
+  }));
 
   // Prior to writing the new files, clean up
   cleanGeneratedCode(args.outputPath);
