@@ -223,30 +223,40 @@ class DiagramFormat(enum.Enum):
 @jsii.data_type(
     jsii_type="@aws/pdk.cdk_graph_plugin_diagram.DiagramOptions",
     jsii_struct_bases=[],
-    name_mapping={"title": "title", "preset": "preset", "theme": "theme"},
+    name_mapping={
+        "title": "title",
+        "node_positions": "nodePositions",
+        "preset": "preset",
+        "theme": "theme",
+    },
 )
 class DiagramOptions:
     def __init__(
         self,
         *,
         title: builtins.str,
+        node_positions: typing.Optional[typing.Mapping[builtins.str, "INodePosition"]] = None,
         preset: typing.Optional[_FilterPreset_47315c13] = None,
         theme: typing.Optional[typing.Union[builtins.str, "IGraphThemeConfigAlt"]] = None,
     ) -> None:
         '''Options for diagrams.
 
         :param title: 
+        :param node_positions: 
         :param preset: 
         :param theme: 
         '''
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__3019720d8f4e085ad4bfbad17e4e6e3436b97f1a0e3718ae6413f430f4ed97a9)
             check_type(argname="argument title", value=title, expected_type=type_hints["title"])
+            check_type(argname="argument node_positions", value=node_positions, expected_type=type_hints["node_positions"])
             check_type(argname="argument preset", value=preset, expected_type=type_hints["preset"])
             check_type(argname="argument theme", value=theme, expected_type=type_hints["theme"])
         self._values: typing.Dict[builtins.str, typing.Any] = {
             "title": title,
         }
+        if node_positions is not None:
+            self._values["node_positions"] = node_positions
         if preset is not None:
             self._values["preset"] = preset
         if theme is not None:
@@ -258,6 +268,13 @@ class DiagramOptions:
         assert result is not None, "Required property 'title' is missing"
         return typing.cast(builtins.str, result)
 
+    @builtins.property
+    def node_positions(
+        self,
+    ) -> typing.Optional[typing.Mapping[builtins.str, "INodePosition"]]:
+        result = self._values.get("node_positions")
+        return typing.cast(typing.Optional[typing.Mapping[builtins.str, "INodePosition"]], result)
+
     @builtins.property
     def preset(self) -> typing.Optional[_FilterPreset_47315c13]:
         result = self._values.get("preset")
@@ -316,7 +333,12 @@ class GraphThemeRenderingIconTarget(enum.Enum):
 @jsii.data_type(
     jsii_type="@aws/pdk.cdk_graph_plugin_diagram.IDiagramConfigBase",
     jsii_struct_bases=[],
-    name_mapping={"filter_plan": "filterPlan", "format": "format", "theme": "theme"},
+    name_mapping={
+        "filter_plan": "filterPlan",
+        "format": "format",
+        "node_positions": "nodePositions",
+        "theme": "theme",
+    },
 )
 class IDiagramConfigBase:
     def __init__(
@@ -324,12 +346,14 @@ class IDiagramConfigBase:
         *,
         filter_plan: typing.Optional[typing.Union[_IGraphFilterPlan_106744ef, typing.Dict[builtins.str, typing.Any]]] = None,
         format: typing.Optional[typing.Union[DiagramFormat, typing.Sequence[DiagramFormat]]] = None,
+        node_positions: typing.Optional[typing.Mapping[builtins.str, "INodePosition"]] = None,
         theme: typing.Optional[typing.Union[builtins.str, "IGraphThemeConfigAlt"]] = None,
     ) -> None:
         '''Base config to specific a unique diagram to be generated.
 
         :param filter_plan: Graph {@link IGraphFilterPlanFilter Plan} used to generate a unique diagram.
         :param format: The output format(s) to generated. Default: ``DiagramFormat.PNG`` - which will through extension also generate ``DiagramFormat.SVG`` and ``DiagramFormat.DOT``
+        :param node_positions: Config for predetermined node positions given their CDK construct id.
         :param theme: Config for graph theme.
         '''
         if isinstance(filter_plan, dict):
@@ -338,12 +362,15 @@ class IDiagramConfigBase:
             type_hints = typing.get_type_hints(_typecheckingstub__5b574db97ce884a6029ce524ba5a0bcf4120c926644bb305c271f9942a4bff53)
             check_type(argname="argument filter_plan", value=filter_plan, expected_type=type_hints["filter_plan"])
             check_type(argname="argument format", value=format, expected_type=type_hints["format"])
+            check_type(argname="argument node_positions", value=node_positions, expected_type=type_hints["node_positions"])
             check_type(argname="argument theme", value=theme, expected_type=type_hints["theme"])
         self._values: typing.Dict[builtins.str, typing.Any] = {}
         if filter_plan is not None:
             self._values["filter_plan"] = filter_plan
         if format is not None:
             self._values["format"] = format
+        if node_positions is not None:
+            self._values["node_positions"] = node_positions
         if theme is not None:
             self._values["theme"] = theme
 
@@ -364,6 +391,14 @@ class IDiagramConfigBase:
         result = self._values.get("format")
         return typing.cast(typing.Optional[typing.Union[DiagramFormat, typing.List[DiagramFormat]]], result)
 
+    @builtins.property
+    def node_positions(
+        self,
+    ) -> typing.Optional[typing.Mapping[builtins.str, "INodePosition"]]:
+        '''Config for predetermined node positions given their CDK construct id.'''
+        result = self._values.get("node_positions")
+        return typing.cast(typing.Optional[typing.Mapping[builtins.str, "INodePosition"]], result)
+
     @builtins.property
     def theme(
         self,
@@ -578,6 +613,40 @@ class _IGraphThemeRenderingOptionsProxy:
 typing.cast(typing.Any, IGraphThemeRenderingOptions).__jsii_proxy_class__ = lambda : _IGraphThemeRenderingOptionsProxy
 
 
+@jsii.interface(jsii_type="@aws/pdk.cdk_graph_plugin_diagram.INodePosition")
+class INodePosition(typing_extensions.Protocol):
+    '''Positional coordinates for a node in inches.'''
+
+    @builtins.property
+    @jsii.member(jsii_name="x")
+    def x(self) -> jsii.Number:
+        ...
+
+    @builtins.property
+    @jsii.member(jsii_name="y")
+    def y(self) -> jsii.Number:
+        ...
+
+
+class _INodePositionProxy:
+    '''Positional coordinates for a node in inches.'''
+
+    __jsii_type__: typing.ClassVar[str] = "@aws/pdk.cdk_graph_plugin_diagram.INodePosition"
+
+    @builtins.property
+    @jsii.member(jsii_name="x")
+    def x(self) -> jsii.Number:
+        return typing.cast(jsii.Number, jsii.get(self, "x"))
+
+    @builtins.property
+    @jsii.member(jsii_name="y")
+    def y(self) -> jsii.Number:
+        return typing.cast(jsii.Number, jsii.get(self, "y"))
+
+# Adding a "__jsii_proxy_class__(): typing.Type" function to the interface
+typing.cast(typing.Any, INodePosition).__jsii_proxy_class__ = lambda : _INodePositionProxy
+
+
 @jsii.data_type(
     jsii_type="@aws/pdk.cdk_graph_plugin_diagram.IPluginConfig",
     jsii_struct_bases=[],
@@ -637,6 +706,7 @@ class IPluginConfig:
     name_mapping={
         "filter_plan": "filterPlan",
         "format": "format",
+        "node_positions": "nodePositions",
         "theme": "theme",
         "name": "name",
         "title": "title",
@@ -649,6 +719,7 @@ class IDiagramConfig(IDiagramConfigBase):
         *,
         filter_plan: typing.Optional[typing.Union[_IGraphFilterPlan_106744ef, typing.Dict[builtins.str, typing.Any]]] = None,
         format: typing.Optional[typing.Union[DiagramFormat, typing.Sequence[DiagramFormat]]] = None,
+        node_positions: typing.Optional[typing.Mapping[builtins.str, INodePosition]] = None,
         theme: typing.Optional[typing.Union[builtins.str, IGraphThemeConfigAlt]] = None,
         name: builtins.str,
         title: builtins.str,
@@ -658,6 +729,7 @@ class IDiagramConfig(IDiagramConfigBase):
 
         :param filter_plan: Graph {@link IGraphFilterPlanFilter Plan} used to generate a unique diagram.
         :param format: The output format(s) to generated. Default: ``DiagramFormat.PNG`` - which will through extension also generate ``DiagramFormat.SVG`` and ``DiagramFormat.DOT``
+        :param node_positions: Config for predetermined node positions given their CDK construct id.
         :param theme: Config for graph theme.
         :param name: Name of the diagram. Used as the basename of the generated file(s) which gets the extension appended.
         :param title: The title of the diagram.
@@ -669,6 +741,7 @@ class IDiagramConfig(IDiagramConfigBase):
             type_hints = typing.get_type_hints(_typecheckingstub__30354dea79562b734bd6bd3e6c15fccb355105edbf147fdb0790dcca3730ddb5)
             check_type(argname="argument filter_plan", value=filter_plan, expected_type=type_hints["filter_plan"])
             check_type(argname="argument format", value=format, expected_type=type_hints["format"])
+            check_type(argname="argument node_positions", value=node_positions, expected_type=type_hints["node_positions"])
             check_type(argname="argument theme", value=theme, expected_type=type_hints["theme"])
             check_type(argname="argument name", value=name, expected_type=type_hints["name"])
             check_type(argname="argument title", value=title, expected_type=type_hints["title"])
@@ -681,6 +754,8 @@ class IDiagramConfig(IDiagramConfigBase):
             self._values["filter_plan"] = filter_plan
         if format is not None:
             self._values["format"] = format
+        if node_positions is not None:
+            self._values["node_positions"] = node_positions
         if theme is not None:
             self._values["theme"] = theme
         if ignore_defaults is not None:
@@ -703,6 +778,14 @@ class IDiagramConfig(IDiagramConfigBase):
         result = self._values.get("format")
         return typing.cast(typing.Optional[typing.Union[DiagramFormat, typing.List[DiagramFormat]]], result)
 
+    @builtins.property
+    def node_positions(
+        self,
+    ) -> typing.Optional[typing.Mapping[builtins.str, INodePosition]]:
+        '''Config for predetermined node positions given their CDK construct id.'''
+        result = self._values.get("node_positions")
+        return typing.cast(typing.Optional[typing.Mapping[builtins.str, INodePosition]], result)
+
     @builtins.property
     def theme(
         self,
@@ -784,6 +867,7 @@ __all__ = [
     "IGraphThemeRendering",
     "IGraphThemeRenderingIconProps",
     "IGraphThemeRenderingOptions",
+    "INodePosition",
     "IPluginConfig",
 ]
 
@@ -825,6 +909,7 @@ def _typecheckingstub__a2478a8d6de8d0b65c7e28e6ee42176f82ac6fa77bb6a501926430454
 def _typecheckingstub__3019720d8f4e085ad4bfbad17e4e6e3436b97f1a0e3718ae6413f430f4ed97a9(
     *,
     title: builtins.str,
+    node_positions: typing.Optional[typing.Mapping[builtins.str, INodePosition]] = None,
     preset: typing.Optional[_FilterPreset_47315c13] = None,
     theme: typing.Optional[typing.Union[builtins.str, IGraphThemeConfigAlt]] = None,
 ) -> None:
@@ -835,6 +920,7 @@ def _typecheckingstub__5b574db97ce884a6029ce524ba5a0bcf4120c926644bb305c271f9942
     *,
     filter_plan: typing.Optional[typing.Union[_IGraphFilterPlan_106744ef, typing.Dict[builtins.str, typing.Any]]] = None,
     format: typing.Optional[typing.Union[DiagramFormat, typing.Sequence[DiagramFormat]]] = None,
+    node_positions: typing.Optional[typing.Mapping[builtins.str, INodePosition]] = None,
     theme: typing.Optional[typing.Union[builtins.str, IGraphThemeConfigAlt]] = None,
 ) -> None:
     """Type checking stubs"""
@@ -852,6 +938,7 @@ def _typecheckingstub__30354dea79562b734bd6bd3e6c15fccb355105edbf147fdb0790dcca3
     *,
     filter_plan: typing.Optional[typing.Union[_IGraphFilterPlan_106744ef, typing.Dict[builtins.str, typing.Any]]] = None,
     format: typing.Optional[typing.Union[DiagramFormat, typing.Sequence[DiagramFormat]]] = None,
+    node_positions: typing.Optional[typing.Mapping[builtins.str, INodePosition]] = None,
     theme: typing.Optional[typing.Union[builtins.str, IGraphThemeConfigAlt]] = None,
     name: builtins.str,
     title: builtins.str,
